"""
Url router for the administration site
"""

from django.contrib import admin
from django.urls import re_path

from core_main_app.admin import core_admin_site
from core_explore_periodic_table_app.components.persistent_query_periodic_table.admin_site import (
    CustomPersistentQueryPeriodicTableAdmin,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)
from core_explore_periodic_table_app.components.search_operator_mapping.admin_site import (
    CustomSearchOperatorMappingAdmin,
)
from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)
from core_explore_periodic_table_app.views.admin import views as admin_views


admin_urls = [
    re_path(
        r"^periodic_table$",
        admin_views.manage_periodic_table_index,
        name="manage_periodic_table_index",
    ),
]

admin.site.register(SearchOperatorMapping, CustomSearchOperatorMappingAdmin)
admin.site.register(
    PersistentQueryPeriodicTable, CustomPersistentQueryPeriodicTableAdmin
)
urls = core_admin_site.get_urls()
core_admin_site.get_urls = lambda: admin_urls + urls



""" Url router for the Explore Period Table
"""

from django.conf.urls import include
from django.urls import re_path

from core_explore_periodic_table_app.views.user import (
    views as user_views,
    ajax as user_ajax,
)

urlpatterns = [
    re_path(r"^rest/", include("core_explore_periodic_table_app.rest.urls")),
    re_path(
        r"^$",
        user_views.PeriodicTableBuildQueryView.as_view(),
        name="core_explore_periodic_table_index",
    ),
    re_path(
        r"^(?P<query_id>\w+)$",
        user_views.PeriodicTableBuildQueryView.as_view(),
        name="core_explore_periodic_table_index",
    ),
    re_path(
        r"^results-redirect",
        user_views.ResultQueryRedirectPeriodicSearchView.as_view(),
        name="core_explore_periodic_table_redirect",
    ),
    re_path(
        r"^get-persistent-query-url$",
        user_ajax.CreatePersistentQueryUrlPeriodicTableView.as_view(),
        name="core_explore_periodic_table_get_persistent_query_url",
    ),
]



""" Integration Test Persistent Query Periodic Table
"""

from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock

from core_explore_periodic_table_app.rest.search_operator_mapping import (
    views as search_operator_mapping_views,
)
from tests.rest.search_operators_mapping.fixtures.fixtures import (
    SearchOperatorMappingFixtures,
)

fixture_search_operator_mapping = SearchOperatorMappingFixtures()


class TestSearchOperatorMapping(IntegrationBaseTestCase):
    """Test Search Operator Mapping"""

    fixture = fixture_search_operator_mapping

    def test_get_all_search_operator_mapping_return_collection(self):
        """test_get_all_search_operator_mapping_return_collection"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
        )

        # Assert
        self.assertListEqual(
            response.data, self.fixture.search_operator_mapping_collection
        )

    def test_get_search_operator_mapping_return_object(self):
        """test_get_search_operator_mapping_return_object"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(
            response.data, self.fixture.search_operator_mapping_1_serialized
        )

    def test_get_search_operator_mapping_with_wrong_param(self):
        """test_get_search_operator_mapping_with_wrong_param"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": "123456789012345678901234"},
        )

        # Assert
        self.assertEqual(response.status_code, 500)

    def test_add_search_operator_mapping(self):
        """test_add_search_operator_mapping"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": str(self.fixture.search_operator_1.id)},
        )

        # Assert
        self.assertEqual(
            response.data["search_operator"], self.fixture.search_operator_1.id
        )

    def test_add_wrong_search_operator_mapping(self):
        """test_add_wrong_search_operator_mapping"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": -1},
        )

        # Assert
        self.assertEqual(response.status_code, 400)

    def test_update_search_operator_mapping(self):
        """test_update_search_operator_mapping"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
            data={"search_operator": str(self.fixture.search_operator_3.id)},
        )

        # Assert
        self.assertEqual(
            response.data["search_operator"], self.fixture.search_operator_3.id
        )

    def test_delete_search_operator_mapping(self):
        """test_delete_search_operator_mapping"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_delete(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 204)



"""Serializers used throughout the Search operator mapping Rest API
"""

from rest_framework.serializers import ModelSerializer

from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)
from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)


class SearchOperatorMappingSerializer(ModelSerializer):
    """SearchOperatorMapping serializer"""

    class Meta:
        """Meta"""

        model = SearchOperatorMapping
        fields = [
            "id",
            "search_operator",
        ]
        read_only_fields = ("id",)

    def create(self, validated_data):
        """Create and return a new `SearchOperatorMapping` instance."""

        new_search_operator_id = validated_data["search_operator"]

        # Create instance from the validated data and insert it in DB
        instance = SearchOperatorMapping(
            search_operator=new_search_operator_id
        )
        search_operator_mapping_api.upsert(instance)

        return instance

    def update(self, instance, validated_data):
        """Update and return an existing `SearchOperatorMapping` instance."""

        new_search_operator_id = validated_data.get(
            "search_operator", instance.search_operator
        )

        instance.search_operator = new_search_operator_id

        return search_operator_mapping_api.upsert(instance)



""" REST views for the Search operators Mapping API
"""

from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions
from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)
from core_explore_periodic_table_app.rest.search_operator_mapping.serializers import (
    SearchOperatorMappingSerializer,
)


class SearchOperatorsMapping(APIView):
    """List the Search Operators Mapping."""

    permission_classes = (IsAdminUser,)
    serializer = SearchOperatorMappingSerializer

    def get(self, request):
        """Get all the search operator mapping object

        Url Parameters:

        Examples:

            ../search-operators-mapping/

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of search operator mapping
            - code: 500
              content: Internal server error
        """
        try:

            # Get objects
            search_operator_mapping_object_list = (
                search_operator_mapping_api.get_all()
            )

            # Serialize object
            search_operator_mapping_serializer = self.serializer(
                search_operator_mapping_object_list, many=True
            )

            # Return response
            return Response(
                search_operator_mapping_serializer.data,
                status=status.HTTP_200_OK,
            )
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def post(self, request):
        """Create a search operator mapping

        Url Parameters:

        Examples:

            ../search-operators-mapping/

        Args:

            request: HTTP request

        Returns:

                - code: 201
                  content: Created search operator
                - code: 400
                  content: Validation error / not unique / model error
                - code: 404
                  content: Search operator not found
                - code: 500
                  content: Internal server error
        """
        try:

            # Build serializer
            serializer = SearchOperatorMappingSerializer(data=request.data)

            # Validate data
            serializer.is_valid(raise_exception=True)

            # Save data
            serializer.save()

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist as does_not_exist_exception:
            content = {"message": str(does_not_exist_exception)}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except exceptions.NotUniqueError as not_unique_error:
            content = {"message": str(not_unique_error)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except KeyError as key_error:
            content = {"message": str(key_error)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ApiError as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )
        except Exception as exception:
            content = {"message": str(exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class SearchOperatorMappingDetail(APIView):
    """Search operator mapping detail"""

    permission_classes = (IsAdminUser,)
    serializer = SearchOperatorMappingSerializer

    def get(self, request, pk):
        """Retrieve search operator mapping from database

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:
            SearchOperatorMapping object
        """
        try:
            # Get object
            search_operator_mapping = search_operator_mapping_api.get_by_id(
                pk=pk
            )

            # Serialize object
            serializer = SearchOperatorMappingSerializer(
                search_operator_mapping
            )

            # Return response
            return Response(serializer.data)
        except exceptions.DoesNotExist:
            content = {"message": "Search operator mapping not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def patch(self, request, pk):
        """Update a search operator mapping

        Parameters:
            {
                "search_operator_id": <search_operator_id>
            }

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 200
              content: Updated data
            - code: 400
              content: Validation error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            search_operator_mapping = search_operator_mapping_api.get_by_id(
                pk=pk
            )

            # Build serializer
            search_operator_mapping_serializer = (
                SearchOperatorMappingSerializer(
                    instance=search_operator_mapping,
                    data=request.data,
                    partial=True,
                )
            )

            # Validate and save search operator
            search_operator_mapping_serializer.is_valid(raise_exception=True)
            search_operator_mapping_serializer.save()

            return Response(
                search_operator_mapping_serializer.data,
                status=status.HTTP_200_OK,
            )
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Search operator not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def delete(self, request, pk):
        """Delete a search operator mapping

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 204
              content: Deletion success
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            search_operator = search_operator_mapping_api.get_by_id(pk=pk)

            # delete object
            search_operator_mapping_api.delete(search_operator)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except exceptions.DoesNotExist:
            content = {"message": "Workspace not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Core Periodic table apps config
"""

import sys

from django.apps import AppConfig

from core_explore_periodic_table_app.permissions import discover


class CoreExplorePeriodicTableAppConfig(AppConfig):
    """Periodic Table configuration"""

    name = "core_explore_periodic_table_app"
    verbose_name = "Core Explore by Periodic Table App"

    def ready(self):
        """Run when the app is ready.

        Returns:

        """
        if "migrate" not in sys.argv:
            discover.init_permissions(self.apps)



"""Explore periodic table app Ajax views
"""

from core_explore_common_app.views.user.ajax import (
    CreatePersistentQueryUrlView,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)


class CreatePersistentQueryUrlPeriodicTableView(CreatePersistentQueryUrlView):
    """Create the persistent url from a Query"""

    view_to_reverse = "core_explore_periodic_table_redirect"

    @staticmethod
    def _create_persistent_query(query):
        # create the persistent query
        return PersistentQueryPeriodicTable(
            user_id=query.user_id,
            content=query.content,
            data_sources=query.data_sources,
        )



"""Url router for the REST API
"""

from django.urls import re_path
from rest_framework.urlpatterns import format_suffix_patterns

from core_explore_periodic_table_app.rest.persistent_query_periodic_table import (
    views as persistent_query_periodic_table_views,
)
from core_explore_periodic_table_app.rest.search_operator_mapping import (
    views as search_operators_mapping_views,
)

urlpatterns = [
    re_path(
        r"^search_operators_mapping/$",
        search_operators_mapping_views.SearchOperatorsMapping.as_view(),
        name="core_explore_periodic_table_app_rest_search_operators_mapping",
    ),
    re_path(
        r"^search_operators_mapping/(?P<pk>\w+)/$",
        search_operators_mapping_views.SearchOperatorMappingDetail.as_view(),
        name="core_explore_periodic_table_app_rest_search_operators_mapping_detail",
    ),
    re_path(
        r"^admin/persistent_query_periodic_table/$",
        persistent_query_periodic_table_views.AdminPersistentQueryPeriodicTableList.as_view(),
        name="core_explore_periodic_table_app_rest_persistent_query_periodic_table_admin_list",
    ),
    re_path(
        r"^persistent_query_periodic_table/$",
        persistent_query_periodic_table_views.PersistentQueryPeriodicTableList.as_view(),
        name="core_explore_periodic_table_app_rest_persistent_query_periodic_table_list",
    ),
    re_path(
        r"^persistent_query_periodic_table/(?P<pk>\w+)/$",
        persistent_query_periodic_table_views.PersistentQueryPeriodicTableDetail.as_view(),
        name="core_explore_periodic_table_app_rest_persistent_query_periodic_table_detail",
    ),
    re_path(
        r"^persistent_query_periodic_table/name/(?P<name>\w+)/$",
        persistent_query_periodic_table_views.PersistentQueryPeriodicTableByName.as_view(),
        name="core_explore_periodic_table_app_rest_persistent_query_periodic_table_name",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



""" Periodic table's views
"""

from typing import Dict, Any, List

from django.urls import reverse
from django.utils.decorators import method_decorator

import core_main_app.components.template_version_manager.api as template_version_manager_api
import core_main_app.utils.decorators as decorators
from core_explore_common_app.components.query import api as query_api
from core_explore_common_app.views.user.views import (
    ResultsView,
    ResultQueryRedirectView,
)
from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_keyword_app.views.user.views import KeywordSearchView
from core_explore_periodic_table_app.components.persistent_query_periodic_table import (
    api as persistent_query_periodic_table_api,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)
from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)
from core_explore_periodic_table_app.permissions import rights
from core_explore_periodic_table_app.views.user.form import PeriodicTableForm
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template import api as template_api
from core_main_app.settings import DATA_SORTING_FIELDS
from core_main_app.utils.rendering import render


class PeriodicTableBuildQueryView(KeywordSearchView):
    """Periodic Table Build Query View"""

    results_url = "core_explore_periodic_table_results"
    query_builder_interface = (
        "core_explore_periodic_table_app/user/periodic_table/periodic.html"
    )

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_PERIODIC_TABLE_CONTENT_TYPE,
            permission=rights.EXPLORE_PERIODIC_TABLE_ACCESS,
        )
    )
    def get(self, request, **kwargs):
        """GET

        Args:
            request:
            **kwargs:

        Returns:

        """
        query_id = str(kwargs["query_id"]) if "query_id" in kwargs else None

        # assets / modals / forms
        context = self._get(request, query_id)

        # Set page title
        context.update({"page_title": "Search By Periodic Table"})

        return render(
            request,
            "core_explore_periodic_table_app/user/index.html",
            assets=self.assets,
            modals=self.modals,
            context=context,
        )

    def _get(self, request, query_id):
        """Prepare the GET context

        Args:
            query_id:

        Returns:

        """
        error = None
        # set the correct default ordering for the context
        default_order = ",".join(DATA_SORTING_FIELDS)
        if query_id is None:
            # create query
            query = query_api.create_default_query(request, [])
            # upsert the query
            query_api.upsert(query, request.user)
            # create all data for select values in forms
            periodic_table_data_form = {
                "query_id": str(query.id),
                "user_id": query.user_id,
            }
        else:  # query_id is not None
            try:
                # get the query id
                query = query_api.get_by_id(query_id, request.user)
                user_id = query.user_id

                # get all elements back
                elements = self._parse_query(query.content)

                # get all version managers
                version_managers = []
                for template in query.templates.all():
                    version_managers.append(str(template.version_manager.id))
                # create all data for select values in forms
                periodic_table_data_form = {
                    "query_id": str(query.id),
                    "user_id": user_id,
                    "elements": elements,
                    "global_templates": version_managers,
                    "order_by_field": super().build_sorting_context_array(
                        query
                    ),
                    "user_templates": version_managers,
                }
                # set the correct ordering for the context
                if periodic_table_data_form["order_by_field"] != 0:
                    default_order = periodic_table_data_form["order_by_field"]
            except Exception as exception:
                error = "An unexpected error occurred while loading the query: {}.".format(
                    str(exception)
                )
                return {"error": error}

        search_form = PeriodicTableForm(
            data=periodic_table_data_form, request=request
        )
        return self._format_keyword_search_context(
            search_form, error, None, default_order
        )

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_PERIODIC_TABLE_CONTENT_TYPE,
            permission=rights.EXPLORE_PERIODIC_TABLE_ACCESS,
        )
    )
    def post(self, request):
        """POST

        Args:
            request:

        Returns:

        """

        # assets / modals / forms
        context = self._post(request)

        # Set page title
        context.update({"page_title": "Search By Periodic Table"})

        return render(
            request,
            "core_explore_periodic_table_app/user/index.html",
            assets=self.assets,
            modals=self.modals,
            context=context,
        )

    def _post(self, request):
        """Prepare the POST context

        Args:
            request:

        Returns:

        """
        error = None
        warning = None
        search_form = PeriodicTableForm(data=request.POST, request=request)
        # validate form
        if search_form.is_valid():
            try:
                # get form values
                query_id = search_form.cleaned_data.get("query_id", None)
                elements = search_form.cleaned_data.get("elements", None)
                global_templates = search_form.cleaned_data.get(
                    "global_templates", []
                )
                user_templates = search_form.cleaned_data.get(
                    "user_templates", []
                )
                order_by_field_array = (
                    search_form.cleaned_data.get("order_by_field", "")
                    .strip()
                    .split(";")
                )
                # get all template version manager ids
                template_version_manager_ids = (
                    global_templates + user_templates
                )
                # from ids, get all version manager
                version_manager_list = (
                    template_version_manager_api.get_by_id_list(
                        template_version_manager_ids, request=request
                    )
                )
                # from all version manager, build a list of all version (template)
                template_ids = []
                list(
                    [
                        template_ids.extend(x.versions)
                        for x in version_manager_list
                    ]
                )
                if query_id is None or elements is None:
                    error = "Expected parameters are not provided"
                else:
                    # get query
                    query = query_api.get_by_id(query_id, request.user)
                    if len(query.data_sources) == 0:
                        warning = "Please select at least 1 data source."
                    else:
                        # update query
                        query.templates.set(
                            template_api.get_all_accessible_by_id_list(
                                template_ids, request=request
                            )
                        )

                        try:
                            element_search_operators = (
                                self._format_keyword_to_search_operators(
                                    elements.split(",")
                                )
                            )
                        except Exception as ex:
                            warning = str(ex)
                            element_search_operators = ""

                        query.content = self._build_query(
                            element_search_operators
                        )
                        # set the data-sources filter value according to the POST request field
                        for data_sources_index in range(
                            len(query.data_sources)
                        ):
                            # update the data-source filter only if it's not a new data-source
                            # (the default filter value is already added when the data-source
                            # is created)
                            if data_sources_index in range(
                                0, len(order_by_field_array)
                            ):
                                query.data_sources[data_sources_index][
                                    "order_by_field"
                                ] = order_by_field_array[data_sources_index]

                        query_api.upsert(query, request.user)
            except DoesNotExist as does_not_exist_error:
                error = (
                    str(does_not_exist_error)
                    if does_not_exist_error
                    else "An unexpected error occurred while retrieving the query."
                )
            except Exception as exception:
                error = "An unexpected error occurred: {}.".format(
                    str(exception)
                )
        else:
            error = "An unexpected error occurred: the form is not valid."

        return self._format_keyword_search_context(
            search_form,
            error,
            warning,
            search_form.cleaned_data.get("order_by_field", "").strip(),
        )

    def _load_assets(self):
        """Return assets structure

        Returns:

        """

        assets = ResultsView._load_assets(self)
        extra_assets: Dict[str, List[Any]] = {
            "js": [
                {
                    "path": "core_explore_periodic_table_app/user/js/periodic_table/periodic.js",
                    "is_raw": False,
                },
                {
                    "path": "core_explore_periodic_table_app/user/js/persistent_query.raw.js",
                    "is_raw": True,
                },
            ],
            "css": [
                "core_explore_keyword_app/user/css/search/search.css",
                "core_explore_periodic_table_app/user/css/periodic_table/periodic.css",
            ],
        }

        assets["js"].extend(extra_assets["js"])
        assets["css"].extend(extra_assets["css"])

        return assets

    def _format_keyword_search_context(
        self, search_form, error, warning, query_order=""
    ):
        """Format the context for the periodic table research page

        Args:
            search_form:
            error:
            warning:
            query_order:

        Returns:

        """
        context = super()._format_keyword_search_context(
            search_form, error, warning, query_order=query_order
        )
        context["results_url"] = "core_explore_periodic_table_results"

        return context

    def _format_keyword_to_search_operators(self, elements_list):
        """Format the elements list to a set of search operators

        Args:
            elements_list: (ex. ["Au","U"...])

        Returns:
            - Search operator list (ex.["so1:Au", "so2:Au", "so1:U", "so2:U"])
            - Empty list if the query is empty
        """
        result = []
        all_so_mapping = search_operator_mapping_api.get_all()

        if all_so_mapping.count() > 0:
            for so_mapping in all_so_mapping:
                search_operator = search_operator_api.get_by_id(
                    str(so_mapping.search_operator.id)
                )
                for element in elements_list:
                    element and result.append(
                        f"{search_operator.name}:{element}"
                    )
        else:
            raise Exception(
                "No search operators has been configured, please contact an administrator."
            )

        return result

    @staticmethod
    def get_description():
        """Get the description page

        Returns:

        """
        return (
            "Click on an element of the Periodic Table to add it to your query. "
            "You can save queries and you will retrieve them on your next connection. "
            "When your query is done, please click on Submit Query to get XML documents"
            " that match the criteria."
        )

    @staticmethod
    def get_title():
        """Get the title page

        Returns:

        """
        return "Periodic Table"


class ResultQueryRedirectPeriodicSearchView(ResultQueryRedirectView):
    """Result Query Redirect Periodic Search View"""

    model_name = PersistentQueryPeriodicTable.__name__
    object_name = "persistent_query_periodic_table"
    redirect_url = "core_explore_periodic_table_index"

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_PERIODIC_TABLE_CONTENT_TYPE,
            permission=rights.EXPLORE_PERIODIC_TABLE_ACCESS,
        )
    )
    def get(self, request, *args, **kwargs):
        return super().get(self, request, *args, **kwargs)

    @staticmethod
    def _get_persistent_query_by_id(persistent_query_id, user):
        return persistent_query_periodic_table_api.get_by_id(
            persistent_query_id, user
        )

    @staticmethod
    def _get_persistent_query_by_name(persistent_query_name, user):
        return persistent_query_periodic_table_api.get_by_name(
            persistent_query_name, user
        )

    @staticmethod
    def get_url_path():
        return reverse(
            ResultQueryRedirectPeriodicSearchView.redirect_url,
            kwargs={"query_id": "query_id"},
        ).split("query_id")[0]

    @staticmethod
    def _get_reversed_url(query):
        return reverse(
            ResultQueryRedirectPeriodicSearchView.redirect_url,
            kwargs={"query_id": query.id},
        )

    @staticmethod
    def _get_reversed_url_if_failed():
        return reverse("core_explore_periodic_table_index")



""" PeriodicTableType api
"""

from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)


def get_all():
    """Return all PeriodicTableType.

    Returns: PeriodicTableType collection
    """
    return SearchOperatorMapping.get_all()


def get_by_id(pk):
    """Get mapping by search operator mapping id

    Args:
        pk:

    Returns:

    """
    return SearchOperatorMapping.get_by_id(pk=pk)


def get_by_search_operator_id(search_operator_id):
    """Get mapping by search operator id

    Args:
        search_operator_id:

    Returns:

    """
    return SearchOperatorMapping.get_by_search_operator_id(
        search_operator_id=search_operator_id
    )


def upsert(search_operator_mapping):
    """Insert or update an search operator mapping

    Args:
        search_operator_mapping:

    Returns:

    """
    search_operator_mapping.save()
    return search_operator_mapping


def delete(search_operator_mapping):
    """Delete a mapping.

    Args:
         search_operator_mapping:

    Returns:
    """
    search_operator_mapping.delete()



""" core explore periodic table admin views
"""

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.http.response import HttpResponseBadRequest

from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)
from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)
from core_explore_periodic_table_app.views.admin.forms import (
    AssociatedPeriodicTableSearchOperatorForm,
)
from core_main_app.utils.rendering import admin_render


@staff_member_required
def manage_periodic_table_index(request):
    """Index admin view

    Args:
        request:

    Returns:

    """
    try:
        if request.method == "POST":
            return _manage_periodic_table_index_post(request)

        return _manage_periodic_table_index_get(request)
    except Exception as exception:
        return HttpResponseBadRequest(str(exception))


def _manage_periodic_table_index_get(request):
    """Index admin view GET

    Args:
        request:

    Returns:

    """
    # load the form
    associated_form = AssociatedPeriodicTableSearchOperatorForm()
    assets = {"css": ["core_explore_periodic_table_app/admin/form.css"]}

    context = {"associated_form": associated_form}

    return admin_render(
        request,
        "core_explore_periodic_table_app/admin/periodic_table_type/manage_periodic_table_type.html",
        context=context,
        assets=assets,
    )


def _manage_periodic_table_index_post(request):
    """Index admin view POST

    Args:
        request:

    Returns:

    """
    # get the new id
    selected_search_operator_list = request.POST.getlist(
        "search_operator_list"
    )
    search_operator_mapping = list(search_operator_mapping_api.get_all())
    search_operator_list = list(search_operator_api.get_all())

    for search_operator in search_operator_list:
        find_in_mapping = next(
            (
                x
                for x in search_operator_mapping
                if x.search_operator.id == search_operator.id
            ),
            None,
        )
        find_in_selected = next(
            (
                y
                for y in selected_search_operator_list
                if y == str(search_operator.id)
            ),
            None,
        )
        if find_in_mapping and not find_in_selected:
            # delete the mapping
            current_search_operator_mapping = (
                search_operator_mapping_api.get_by_search_operator_id(
                    search_operator.id
                )
            )
            search_operator_mapping_api.delete(current_search_operator_mapping)
        elif not find_in_mapping and find_in_selected:
            # add the mapping
            search_operator_mapping_api.upsert(
                SearchOperatorMapping(search_operator=search_operator)
            )

    # load the form
    associated_form = AssociatedPeriodicTableSearchOperatorForm()
    assets = {"css": ["core_explore_periodic_table_app/admin/form.css"]}

    context = {"associated_form": associated_form}

    messages.add_message(request, messages.SUCCESS, "Information saved.")

    return admin_render(
        request,
        "core_explore_periodic_table_app/admin/periodic_table_type/manage_periodic_table_type.html",
        context=context,
        assets=assets,
    )



""" Serializers used for the persistent query periodic table REST API.
"""

from rest_framework.serializers import ModelSerializer

from core_explore_periodic_table_app.components.persistent_query_periodic_table import (
    api as persistent_query_periodic_table_api,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)


class PersistentQueryPeriodicTableSerializer(ModelSerializer):
    """Persistent query periodic_table"""

    class Meta:
        """Meta"""

        model = PersistentQueryPeriodicTable
        fields = ["id", "user_id", "content", "templates", "name"]
        read_only_fields = ("id", "user_id")

    def create(self, validated_data):
        """Create and return a new `PersistentQueryPeriodicTable` instance, given the validated data."""

        # Create instance from the validated data and insert it in DB
        persistent_query_periodic_table = PersistentQueryPeriodicTable(
            user_id=str(self.context["request"].user.id),
            content=(
                validated_data["content"]
                if "content" in validated_data
                else None
            ),
            name=validated_data["name"] if "name" in validated_data else None,
        )

        persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, self.context["request"].user
        )
        if "templates" in validated_data:
            persistent_query_periodic_table.templates.set(
                validated_data["templates"]
            )

        return persistent_query_periodic_table

    # Update instance from the validated data and insert it in DB
    def update(self, persistent_query_periodic_table, validated_data):
        """Update and return an existing `PersistentQueryPeriodicTable` instance, given the validated
        data.
        """
        persistent_query_periodic_table.content = validated_data.get(
            "content", persistent_query_periodic_table.content
        )
        persistent_query_periodic_table.name = validated_data.get(
            "name", persistent_query_periodic_table.name
        )
        persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, self.context["request"].user
        )
        if "templates" in validated_data:
            persistent_query_periodic_table.templates.set(
                validated_data["templates"]
            )
        return persistent_query_periodic_table


class PersistentQueryPeriodicTableAdminSerializer(ModelSerializer):
    """PersistentQueryAdminPeriodicTable Serializer"""

    class Meta:
        """Meta"""

        model = PersistentQueryPeriodicTable
        fields = ["id", "user_id", "content", "templates", "name"]

    def create(self, validated_data):
        """
        Create and return a new `PersistentQueryPeriodicTable` instance, given the validated data.
        """
        # Create data
        persistent_query_periodic_table = PersistentQueryPeriodicTable(
            user_id=validated_data["user_id"],
            content=(
                validated_data["content"]
                if "content" in validated_data
                else None
            ),
            name=validated_data["name"] if "name" in validated_data else None,
        )
        persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, self.context["request"].user
        )
        if "templates" in validated_data:
            persistent_query_periodic_table.templates.set(
                validated_data["templates"]
            )
        return persistent_query_periodic_table



""" Unit Test Persistent Query Periodic Table
"""

from django.contrib.auth.models import AnonymousUser

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user

from core_explore_common_app.settings import (
    CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table import (
    api as persistent_query_periodic_table_api,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)
from tests.components.persistent_query_periodic_table.fixtures.fixtures import (
    PersistentQueryPeriodicTableFixtures,
)

fixture_persistent_query_periodic_table = (
    PersistentQueryPeriodicTableFixtures()
)


class TestPersistentQueryPeriodicTableGetById(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Get By Id"""

    fixture = fixture_persistent_query_periodic_table

    def test_get_by_id_as_superuser_returns_persistent_query_periodic_table(
        self,
    ):
        """test_get_by_id_as_superuser_returns_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table_id = (
            self.fixture.persistent_query_periodic_table_1.id
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_periodic_table = (
            persistent_query_periodic_table_api.get_by_id(
                persistent_query_periodic_table_id, mock_user
            )
        )

        # Assert
        self.assertTrue(
            isinstance(
                persistent_query_periodic_table, PersistentQueryPeriodicTable
            )
        )

    def test_get_by_id_as_owner_returns_persistent_query_periodic_table(self):
        """test_get_by_id_as_owner_returns_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table_id = (
            self.fixture.persistent_query_periodic_table_1.id
        )
        mock_user = create_mock_user("1")

        # Act
        persistent_query_periodic_table = (
            persistent_query_periodic_table_api.get_by_id(
                persistent_query_periodic_table_id, mock_user
            )
        )

        # Assert
        self.assertTrue(
            isinstance(
                persistent_query_periodic_table, PersistentQueryPeriodicTable
            )
        )

    def test_get_by_id_as_anonymous_user(self):
        """test_get_by_id_as_anonymous_user"""

        # Arrange
        persistent_query_periodic_table_id = (
            self.fixture.persistent_query_periodic_table_1.id
        )

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_periodic_table_api.get_by_id(
                        persistent_query_periodic_table_id, AnonymousUser()
                    ),
                    PersistentQueryPeriodicTable,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_periodic_table_api.get_by_id(
                    persistent_query_periodic_table_id, AnonymousUser()
                )

    def test_get_by_id_as_user_not_owner(self):
        """test_get_by_id_as_user_not_owner"""

        # Arrange
        persistent_query_periodic_table_id = (
            self.fixture.persistent_query_periodic_table_1.id
        )
        mock_user = create_mock_user("0")

        # Act # Assert

        self.assertTrue(
            isinstance(
                persistent_query_periodic_table_api.get_by_id(
                    persistent_query_periodic_table_id, mock_user
                ),
                PersistentQueryPeriodicTable,
            )
        )


class TestPersistentQueryPeriodicTableGetByName(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Get By Name"""

    fixture = fixture_persistent_query_periodic_table

    def test_get_by_name_as_superuser_returns_persistent_query_periodic_table(
        self,
    ):
        """test_get_by_name_as_superuser_returns_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table_name = (
            self.fixture.persistent_query_periodic_table_1.name
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_periodic_table = (
            persistent_query_periodic_table_api.get_by_name(
                persistent_query_periodic_table_name, mock_user
            )
        )

        # Assert
        self.assertTrue(
            isinstance(
                persistent_query_periodic_table, PersistentQueryPeriodicTable
            )
        )

    def test_get_by_name_as_owner_returns_persistent_query_periodic_table(
        self,
    ):
        """test_get_by_name_as_owner_returns_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table_name = (
            self.fixture.persistent_query_periodic_table_1.name
        )
        mock_user = create_mock_user("1")

        # Act
        persistent_query_periodic_table = (
            persistent_query_periodic_table_api.get_by_name(
                persistent_query_periodic_table_name, mock_user
            )
        )

        # Assert
        self.assertTrue(
            isinstance(
                persistent_query_periodic_table, PersistentQueryPeriodicTable
            )
        )

    def test_get_by_name_as_user_not_owner(self):
        """test_get_by_name_as_user_not_owner"""

        # Arrange
        persistent_query_periodic_table_name = (
            self.fixture.persistent_query_periodic_table_1.name
        )
        mock_user = create_mock_user("0")

        # Act # Assert
        self.assertTrue(
            isinstance(
                persistent_query_periodic_table_api.get_by_name(
                    persistent_query_periodic_table_name, mock_user
                ),
                PersistentQueryPeriodicTable,
            )
        )

    def test_get_by_name_as_anonymous_user(self):
        """test_get_by_name_as_anonymous_user"""

        # Arrange
        persistent_query_periodic_table_name = (
            self.fixture.persistent_query_periodic_table_1.name
        )

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_periodic_table_api.get_by_name(
                        persistent_query_periodic_table_name, AnonymousUser()
                    ),
                    PersistentQueryPeriodicTable,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_periodic_table_api.get_by_name(
                    persistent_query_periodic_table_name, AnonymousUser()
                )


class TestPersistentQueryPeriodicTableDelete(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Delete"""

    fixture = fixture_persistent_query_periodic_table

    def test_delete_others_as_superuser_deletes_persistent_query_periodic_table(
        self,
    ):
        """test_delete_others_as_superuser_deletes_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_periodic_table_api.delete(
            persistent_query_periodic_table, mock_user
        )

    def test_delete_own_table_deletes_persistent_query_periodic_table(
        self,
    ):
        """test_delete_own_table_deletes_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        mock_user = create_mock_user("1")

        # Act
        persistent_query_periodic_table_api.delete(
            persistent_query_periodic_table, mock_user
        )

    def test_delete_others_as_user_raises_error(self):
        """test_delete_others_as_user_raises_error"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.delete(
                persistent_query_periodic_table, mock_user
            )

    def test_delete_others_as_anonymous_raises_error(
        self,
    ):
        """test_delete_others_as_anonymous_raises_error"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.delete(
                persistent_query_periodic_table, AnonymousUser()
            )


class TestPersistentQueryPeriodicTableUpdate(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Update"""

    fixture = fixture_persistent_query_periodic_table

    def test_update_others_as_superuser_updates_persistent_query_periodic_table(
        self,
    ):
        """test_update_others_as_superuser_updates_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        persistent_query_periodic_table.name = (
            "new_name_persistent_query_periodic_table_1"
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryPeriodicTable))
        self.assertTrue(
            result.name, "new_name_persistent_query_periodic_table_1"
        )

    def test_update_own_updates_persistent_query_periodic_table(
        self,
    ):
        """test_update_own_updates_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        mock_user = create_mock_user("1")
        persistent_query_periodic_table.name = (
            "new_name_persistent_query_periodic_table_1"
        )
        # Act
        result = persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryPeriodicTable))
        self.assertTrue(
            result.name, "new_name_persistent_query_periodic_table_1"
        )

    def test_update_others_as_user_raises_error(self):
        """test_update_others_as_user_raises_error"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )
        persistent_query_periodic_table.name = (
            "new_name_persistent_query_periodic_table_1"
        )
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.upsert(
                persistent_query_periodic_table, mock_user
            )

    def test_update_others_as_anonymous_raises_error(
        self,
    ):
        """test_update_others_as_anonymous_raises_error"""

        # Arrange
        persistent_query_periodic_table = (
            self.fixture.persistent_query_periodic_table_1
        )

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.upsert(
                persistent_query_periodic_table, AnonymousUser()
            )


class TestPersistentQueryPeriodicTableCreate(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Create"""

    fixture = fixture_persistent_query_periodic_table

    def test_create_for_others_as_superuser_creates_persistent_query_periodic_table(
        self,
    ):
        """test_create_for_others_as_superuser_creates_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = PersistentQueryPeriodicTable(
            name="new_persistent_query_periodic_table", user_id="0"
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryPeriodicTable))
        self.assertTrue(result.name, "new_persistent_query_periodic_table")

    def test_create_as_user_creates_persistent_query_periodic_table(
        self,
    ):
        """test_create_as_user_creates_persistent_query_periodic_table"""

        # Arrange
        persistent_query_periodic_table = PersistentQueryPeriodicTable(
            name="new_persistent_query_periodic_table", user_id="1"
        )
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_periodic_table_api.upsert(
            persistent_query_periodic_table, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryPeriodicTable))
        self.assertTrue(result.name, "new_persistent_query_periodic_table")

    def test_create_as_anonymous_user(self):
        """test_create_as_anonymous_user"""

        # Arrange
        persistent_query_periodic_table = PersistentQueryPeriodicTable(
            name="new_persistent_query_periodic_table", user_id="None"
        )

        # Act
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            result = persistent_query_periodic_table_api.upsert(
                persistent_query_periodic_table, AnonymousUser()
            )
            # Assert
            self.assertTrue(isinstance(result, PersistentQueryPeriodicTable))
            self.assertTrue(result.name, "new_persistent_query_periodic_table")

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_periodic_table_api.upsert(
                    persistent_query_periodic_table, AnonymousUser()
                )


class TestPersistentQueryPeriodicTableGetAll(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Get All"""

    fixture = fixture_persistent_query_periodic_table

    def test_get_all_as_superuser_returns_all_persistent_query_periodic_table(
        self,
    ):
        """test_get_all_as_superuser_returns_all_persistent_query_periodic_table"""

        # Arrange
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_periodic_table_api.get_all(mock_user)

        # Assert
        self.assertTrue(len(result), 3)

    def test_get_all_as_user_raises_error(self):
        """test_get_all_as_user_raises_error"""

        # Arrange
        mock_user = create_mock_user("1")

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.get_all(mock_user)

    def test_get_all_as_anonymous_user_raises_error(self):
        """test_get_all_as_anonymous_user_raises_error"""

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.get_all(AnonymousUser())


class TestPersistentQueryPeriodicTableGetAllByUser(IntegrationBaseTestCase):
    """Test Persistent Query Periodic Table Get All By User"""

    fixture = fixture_persistent_query_periodic_table

    def test_get_all_by_user_as_superuser_returns_all_user_persistent_query_periodic_table(
        self,
    ):
        """test_get_all_by_user_as_superuser_returns_all_user_persistent_query_periodic_table"""

        # Arrange
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_periodic_table_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)

    def test_get_all_by_user_returns_all_user_persistent_query_periodic_table(
        self,
    ):
        """test_get_all_by_user_returns_all_user_persistent_query_periodic_table"""

        # Arrange
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_periodic_table_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)

    def test_get_all_as_anonymous_user_raises_error(self):
        """test_get_all_as_anonymous_user_raises_error"""

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_periodic_table_api.get_all_by_user(
                AnonymousUser()
            )



""" Custom admin site for the Search Operator Mapping model
"""

from django.contrib import admin


class CustomSearchOperatorMappingAdmin(admin.ModelAdmin):
    """CustomSearchOperatorMappingAdmin"""

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Search Operator Mapping"""
        return False



""" Unit tests for periodic table admin views
"""

from unittest.case import TestCase

from django.http.request import HttpRequest
from unittest.mock import patch

from core_explore_periodic_table_app.views.admin import views as admin_views


class TestManagePeriodicTableIndexPost(TestCase):
    """Test Manage Periodic Table Index Post"""

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "messages")
    def test_manage_periodic_table_index_post(
        self, mock_messages, mock_admin_render
    ):
        """test_manage_periodic_table_index_post"""

        # Arrange
        request = HttpRequest()
        # Act
        admin_views._manage_periodic_table_index_post(request)
        mock_admin_render.assert_called()



""" Permission Tests for Persistent Query Periodic Table
"""

from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_explore_periodic_table_app.rest.search_operator_mapping import (
    views as search_operator_mapping_views,
)
from tests.rest.search_operators_mapping.fixtures.fixtures import (
    SearchOperatorMappingFixtures,
)

fixture_search_operator_mapping = SearchOperatorMappingFixtures()


class TestSearchOperatorMappingPermissions(IntegrationBaseTestCase):
    """Test Search Operator Mapping Permissions"""

    fixture = fixture_search_operator_mapping

    def test_get_all_staff_return_200(self):
        """test_get_all_staff_return_200"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
        )

        # Assert
        self.assertEqual(response.status_code, 200)

    def test_get_all_anonymous_return_403(self):
        """test_get_all_anonymous_return_403"""

        # Arrange
        user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_get_all_superuser_return_403(self):
        """test_get_all_superuser_return_403"""

        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_get_all_authenticated_return_403(self):
        """test_get_all_authenticated_return_403"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_get_search_operator_mapping_staff_return_200(self):
        """test_get_search_operator_mapping_staff_return_200"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 200)

    def test_get_search_operator_mapping_anonymous_return_403(self):
        """test_get_search_operator_mapping_anonymous_return_403"""

        # Arrange
        user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_get_search_operator_mapping_superuser_return_403(self):
        """test_get_search_operator_mapping_superuser_return_403"""

        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_get_search_operator_mapping_authenticated_return_403(self):
        """test_get_search_operator_mapping_authenticated_return_403"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_add_search_operator_mapping_staff_return_201(self):
        """test_add_search_operator_mapping_staff_return_201"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": str(self.fixture.search_operator_3.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 201)

    def test_add_search_operator_mapping_anonymous_return_403(self):
        """test_add_search_operator_mapping_anonymous_return_403"""

        # Arrange
        user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": str(self.fixture.search_operator_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_add_search_operator_mapping_superuser_return_403(self):
        """test_add_search_operator_mapping_superuser_return_403"""

        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": str(self.fixture.search_operator_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_add_search_operator_mapping_authenticated_return_403(self):
        """test_add_search_operator_mapping_authenticated_return_403"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            search_operator_mapping_views.SearchOperatorsMapping.as_view(),
            user,
            data={"search_operator": str(self.fixture.search_operator_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_update_search_operator_mapping_staff_return_200(self):
        """test_update_search_operator_mapping_staff_return_200"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
            data={"search_operator": str(self.fixture.search_operator_3.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 200)

    def test_update_search_operator_mapping_anonymous_return_403(self):
        """test_update_search_operator_mapping_anonymous_return_403"""

        # Arrange
        user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_patch(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
            data={"search_operator": str(self.fixture.search_operator_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_update_search_operator_mapping_superuser_return_403(self):
        """test_update_search_operator_mapping_superuser_return_403"""

        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_patch(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
            data={"search_operator": str(self.fixture.search_operator_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_update_search_operator_mapping_authenticated_return_403(self):
        """test_update_search_operator_mapping_authenticated_return_403"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
            data={"search_operator": str(self.fixture.search_operator_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_delete_search_operator_mapping_staff_return_204(self):
        """test_delete_search_operator_mapping_staff_return_204"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_delete(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 204)

    def test_delete_search_operator_mapping_anonymous_return_403(self):
        """test_delete_search_operator_mapping_anonymous_return_403"""

        # Arrange
        user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_delete(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_delete_search_operator_mapping_superuser_return_403(self):
        """test_delete_search_operator_mapping_superuser_return_403"""

        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_delete(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)

    def test_delete_search_operator_mapping_authenticated_return_403(self):
        """test_delete_search_operator_mapping_authenticated_return_403"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            search_operator_mapping_views.SearchOperatorMappingDetail.as_view(),
            user,
            param={"pk": self.fixture.search_operator_mapping_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, 403)



""" Fixtures files for Persistent Query Periodic Table
"""

from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)

from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)


class PersistentQueryPeriodicTableFixtures(FixtureInterface):
    """Persistent query periodic_table fixtures"""

    persistent_query_periodic_table_1 = None
    persistent_query_periodic_table_2 = None
    persistent_query_periodic_table_3 = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_query_collection()

    def generate_query_collection(self):
        """Generate a PePersistentQueryPeriodicTable collection.

        Returns:

        """

        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.persistent_query_periodic_table_1 = PersistentQueryPeriodicTable(
            user_id="1", name="persistent_query_periodic_table_1"
        )

        self.persistent_query_periodic_table_1.save()

        self.persistent_query_periodic_table_2 = PersistentQueryPeriodicTable(
            user_id="2", name="persistent_query_periodic_table_2"
        )
        self.persistent_query_periodic_table_2.save()

        self.persistent_query_periodic_table_3 = PersistentQueryPeriodicTable(
            user_id="None", name="persistent_query_periodic_table_3"
        )
        self.persistent_query_periodic_table_3.save()

        self.data_collection = [
            self.persistent_query_periodic_table_1,
            self.persistent_query_periodic_table_2,
            self.persistent_query_periodic_table_3,
        ]



""" Periodic Table Model
"""

from django.db import models

from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)


class SearchOperatorMapping(models.Model):
    """Periodic Table Type configuration object"""

    search_operator = models.ForeignKey(
        SearchOperator, on_delete=models.CASCADE, blank=False
    )

    @staticmethod
    def get_all():
        """Get all SearchOperatorMapping.

        Args:

        Returns:

        """
        return SearchOperatorMapping.objects.all()

    @staticmethod
    def get_by_id(pk):
        """Get search operator mapping by id

        Args:
            pk:

        Returns:

        """
        return SearchOperatorMapping.objects.get(pk=str(pk))

    @staticmethod
    def get_by_search_operator_id(search_operator_id):
        """Get mapping by search operator id

        Args:
            search_operator_id:

        Returns:

        """
        return SearchOperatorMapping.objects(
            search_operator=str(search_operator_id)
        )



""" Explore by periodic table form
"""

from django import forms
from django.core.exceptions import ValidationError

from core_main_app.commons.exceptions import QueryError
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.utils.query.mongo.prepare import sanitize_value


class PeriodicTableForm(forms.Form):
    """
    Search by periodic table form
    """

    elements = forms.CharField(widget=forms.HiddenInput(), required=False)
    query_id = forms.CharField(widget=forms.HiddenInput())
    user_id = forms.CharField(widget=forms.HiddenInput())
    global_templates = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple(), required=False
    )
    user_templates = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple(), required=False
    )
    order_by_field = forms.CharField(
        required=False, widget=forms.HiddenInput()
    )

    def __init__(self, *args, **kwargs):
        """Init Periodic table form"""
        request = kwargs.pop("request")
        super().__init__(*args, **kwargs)

        # initialize template filters
        global_templates = [
            (template_version_manager.id, template_version_manager.title)
            for template_version_manager in template_version_manager_api.get_active_global_version_manager(
                request=request
            )
        ]
        user_templates = [
            (template_version_manager.id, template_version_manager.title)
            for template_version_manager in template_version_manager_api.get_active_version_manager_by_user_id(
                request=request
            )
        ]
        self.fields["global_templates"].choices = global_templates
        self.fields["user_templates"].choices = user_templates

    def clean_elements(self):
        try:
            return sanitize_value(self.cleaned_data["elements"])
        except QueryError as error:
            raise ValidationError(str(error))



""" Url router for the explore periodic table application
"""

from django.conf.urls import include
from django.urls import re_path

from core_main_app.admin import core_admin_site

urlpatterns = [
    re_path(r"^core-admin/", core_admin_site.urls),
    re_path(r"^", include("core_main_app.urls")),
    re_path(r"^", include("core_explore_periodic_table_app.urls")),
]



""" Persistent Query Periodic table API
"""

from core_explore_common_app.access_control.api import (
    can_read_persistent_query,
    can_write_persistent_query,
)
from core_explore_periodic_table_app.components.persistent_query_periodic_table.models import (
    PersistentQueryPeriodicTable,
)
from core_main_app.access_control.api import has_perm_administration
from core_main_app.access_control.decorators import access_control


@access_control(can_write_persistent_query)
def upsert(persistent_query_periodic_table, user):
    """Saves or update persistent query

    Args:
        persistent_query_periodic_table:
        user:

    Returns:

    """
    persistent_query_periodic_table.save()
    return persistent_query_periodic_table


@access_control(can_read_persistent_query)
def get_by_id(persistent_query_periodic_table_id, user):
    """Return the Persistent Query Periodic Table with the given id

    Args:
        persistent_query_periodic_table_id:
        user:

    Returns:

    """
    return PersistentQueryPeriodicTable.get_by_id(
        persistent_query_periodic_table_id
    )


@access_control(can_read_persistent_query)
def get_by_name(persistent_query_periodic_table_name, user):
    """Return the Persistent Query Periodic Table with the given name

    Args:
        persistent_query_periodic_table_name:
        user:
    Returns:

    """
    return PersistentQueryPeriodicTable.get_by_name(
        persistent_query_periodic_table_name
    )


@access_control(can_write_persistent_query)
def delete(persistent_query_periodic_table, user):
    """Deletes the Persistent Query Periodic Table and the element associated

    Args:
        persistent_query_periodic_table:
        user:
    """
    persistent_query_periodic_table.delete()


@access_control(can_write_persistent_query)
def set_name(persistent_query_periodic_table, name, user):
    """Set name to Persistent Query Periodic Table

    Args:
        persistent_query_periodic_table:
        name:
        user:
    """
    persistent_query_periodic_table.name = name
    persistent_query_periodic_table.save()


@access_control(has_perm_administration)
def get_all(user):
    """get all Persistent Query Periodic Table

    Args:
        user:
    """
    return PersistentQueryPeriodicTable.get_all()


@access_control(can_read_persistent_query)
def get_all_by_user(user):
    """get persistent Query Periodic Table by user

    Args:
        user:
    """
    return PersistentQueryPeriodicTable.get_all_by_user(user.id)



""" Add Explore Period Table in main menu
"""

from django.urls import reverse
from menu import Menu, MenuItem

Menu.add_item(
    "explorer",
    MenuItem(
        "Query by Periodic Table", reverse("core_explore_periodic_table_index")
    ),
),

periodic_table_children = (
    MenuItem(
        "Settings",
        reverse("core-admin:manage_periodic_table_index"),
        icon="table",
    ),
)

Menu.add_item(
    "admin", MenuItem("PERIODIC TABLE", None, children=periodic_table_children)
)



""" Fixtures files for Data
"""

from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)
from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)
from core_explore_periodic_table_app.rest.search_operator_mapping.serializers import (
    SearchOperatorMappingSerializer,
)


class SearchOperatorMappingFixtures(FixtureInterface):
    """Search Operator Mapping fixture"""

    search_operator_mapping_1 = None
    search_operator_mapping_2 = None
    search_operator_mapping_3 = None

    search_operator_mapping_1_serialized = None
    search_operator_mapping_2_serialized = None
    search_operator_mapping_3_serialized = None

    search_operator_1 = None
    search_operator_2 = None
    search_operator_3 = None

    def insert_data(self):
        """Insert a set of Search Operator Mapping.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_search_operator_collection()
        self.generate_search_operator_mapping_collection()

    def generate_search_operator_collection(self):
        """Generate a Search Operator collection.

        Returns:

        """
        self.search_operator_1 = SearchOperator(
            name="name1", xpath_list=["a/b/c"], dot_notation_list=["a.b.c"]
        )
        self.search_operator_1.save()

        self.search_operator_2 = SearchOperator(
            name="name2", xpath_list=["d/e/f"], dot_notation_list=["d.e.f"]
        )
        self.search_operator_2.save()

        self.search_operator_3 = SearchOperator(
            name="name3", xpath_list=["g/h/i"], dot_notation_list=["g.h.i"]
        )
        self.search_operator_3.save()

        self.search_operator_collection = [
            self.search_operator_1,
            self.search_operator_2,
            self.search_operator_3,
        ]

    def generate_search_operator_mapping_collection(self):
        """Generate a Search Operator Mapping collection.

        Returns:

        """
        self.search_operator_mapping_1 = SearchOperatorMapping(
            search_operator=self.search_operator_1
        )
        self.search_operator_mapping_1.save()

        self.search_operator_mapping_2 = SearchOperatorMapping(
            search_operator=self.search_operator_2
        )
        self.search_operator_mapping_2.save()

        serializer = SearchOperatorMappingSerializer(
            data=[
                self.search_operator_mapping_1,
                self.search_operator_mapping_2,
            ],
            many=True,
        )

        serializer.is_valid()

        self.search_operator_mapping_collection = serializer.data

        self.search_operator_mapping_1_serialized = (
            self.search_operator_mapping_collection[0]
        )
        self.search_operator_mapping_2_serialized = (
            self.search_operator_mapping_collection[1]
        )



""" Unit tests for SearchOperatorMapping API calls.
"""

from unittest import TestCase, mock

from core_main_app.commons import exceptions
from core_explore_keyword_app.components.search_operator.models import (
    SearchOperator,
)
from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)
from core_explore_periodic_table_app.components.search_operator_mapping.models import (
    SearchOperatorMapping,
)


class TestsApiGetAll(TestCase):
    """Test Api Get All"""

    @mock.patch.object(SearchOperatorMapping, "get_all")
    def test_returns_no_error(self, mock_get_all):
        """test_returns_no_error"""

        expected_result = [
            "search_operator_mapping1",
            "search_operator_mapping2",
        ]
        mock_get_all.return_value = expected_result

        self.assertListEqual(
            search_operator_mapping_api.get_all(), expected_result
        )


class TestsApiGetById(TestCase):
    """Test Api Get By Id"""

    @mock.patch.object(SearchOperatorMapping, "get_by_id")
    def test_returns_no_error(self, mock_get_by_id):
        """test_returns_no_error"""

        expected_result = "search_operator_mapping"
        mock_get_by_id.return_value = expected_result

        self.assertEqual(
            search_operator_mapping_api.get_by_id("mock_id"), expected_result
        )

    @mock.patch.object(SearchOperatorMapping, "get_by_id")
    def test_incorrect_id_raises_api_error(self, mock_get_by_id):
        """test_incorrect_id_raises_api_error"""

        mock_get_by_id.side_effect = exceptions.ModelError(
            message="mock error"
        )

        with self.assertRaises(exceptions.ModelError):
            search_operator_mapping_api.get_by_id("mock_id")

    @mock.patch.object(SearchOperatorMapping, "get_by_id")
    def test_not_exist_raises_api_error(self, mock_get_by_id):
        """test_not_exist_raises_api_error"""

        mock_get_by_id.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            search_operator_mapping_api.get_by_id("mock_id")


class TestsApiGetByName(TestCase):
    """Test Api Get By Name"""

    @mock.patch.object(SearchOperatorMapping, "get_by_search_operator_id")
    def test_returns_no_error(self, get_by_search_operator_id):
        """test_returns_no_error"""

        expected_result = "search_operator_mapping"
        get_by_search_operator_id.return_value = expected_result

        self.assertEqual(
            search_operator_mapping_api.get_by_search_operator_id("mock_id"),
            expected_result,
        )

    @mock.patch.object(SearchOperatorMapping, "get_by_search_operator_id")
    def test_not_exist_raises_api_error(self, get_by_search_operator_id):
        """test_not_exist_raises_api_error"""

        get_by_search_operator_id.side_effect = exceptions.DoesNotExist(
            message="mock error"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            search_operator_mapping_api.get_by_search_operator_id("mock_name")


class TestsApiUpsert(TestCase):
    """Test Api Upsert"""

    def setUp(self) -> None:
        self.mock_search_operator_mapping = SearchOperatorMapping(
            search_operator=SearchOperator(
                name="mock_operator", xpath_list=["/x/path/a", "/x/path/b"]
            )
        )

    @mock.patch.object(SearchOperatorMapping, "save")
    def test_returns_no_error(self, mock_save):
        """test_returns_no_error"""

        mock_save.return_value = None
        search_operator_mapping_api.upsert(self.mock_search_operator_mapping)

    @mock.patch.object(SearchOperatorMapping, "save")
    def test_duplicate_raises_api_error(self, mock_save):
        """test_duplicate_raises_api_error"""

        mock_save.side_effect = exceptions.NotUniqueError(message="mock error")

        with self.assertRaises(exceptions.NotUniqueError):
            search_operator_mapping_api.upsert(
                self.mock_search_operator_mapping
            )


class TestsApiDelete(TestCase):
    """Test Api Delete"""

    @mock.patch.object(SearchOperatorMapping, "delete")
    def test_returns_no_error(self, mock_delete):
        """test_returns_no_error"""

        mock_delete.return_value = None

        self.assertEqual(
            search_operator_mapping_api.delete(SearchOperatorMapping()), None
        )



""" Forms admin Periodic Table
"""

from django import forms

from core_explore_keyword_app.components.search_operator import (
    api as search_operator_api,
)
from core_explore_periodic_table_app.components.search_operator_mapping import (
    api as search_operator_mapping_api,
)


def _initial_form():
    """Set the initial state of the form.

    Returns:
        List of fields select.

    """
    search_operator_mapping = search_operator_mapping_api.get_all()
    result = []

    for mapping in search_operator_mapping:
        result.append(str(mapping.search_operator.id))

    return result


def _get_search_operators():
    """Get types versions.

    Returns:
        List of types versions.

    """
    type_list = []
    try:
        # display all global types
        all_search_operator_list = search_operator_api.get_all()
        for search_operator in all_search_operator_list:
            type_list.append((search_operator.id, search_operator.name))
    except Exception:
        pass

    return type_list


class AssociatedPeriodicTableSearchOperatorForm(forms.Form):
    """Associated Periodic Table Type form"""

    search_operator_list = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple,
        required=False,
        label="Search operator list",
        error_messages="",
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["search_operator_list"].initial = _initial_form()
        self.fields["search_operator_list"].choices = _get_search_operators()
